package view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.Timer;

public class ProgressDialog extends JDialog{
    
    public final static int OK = 0;
    public final static int CANCELLED = 1;
    public final static int TIME_OUT = 2;
    private volatile int closeReason = CANCELLED;
        
    final JLabel label = new JLabel("Message", JLabel.CENTER);
    
    final JButton button = new JButton(new AbstractAction() {
        @Override
        public void actionPerformed(ActionEvent ae) {
            timer.stop();
            closeReason = CANCELLED;
            setVisible(false);
            dispose();
        }
    });
    
    final Timer timer = new Timer(0, e -> {
        closeReason = TIME_OUT;
        setVisible(false);
        dispose();
    });
    
    public ProgressDialog(JFrame parent){
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        setModal(true);
        setPreferredSize(new Dimension(500, 200));
        setLayout(new BorderLayout(10, 10));
        setResizable(false);
        
        button.setText("Cancel");
        label.setText("Message");
        
        GroupLayout layout = new GroupLayout(getContentPane());
        setLayout(layout);
        
        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);
        
        layout.setVerticalGroup(
            layout.createSequentialGroup()
                .addComponent(label, 30, 30, Short.MAX_VALUE)
                .addComponent(button, 30, 30, 30)
        );

        layout.setHorizontalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.CENTER)
                .addComponent(label, 30, 200, Short.MAX_VALUE)
                .addComponent(button, 100, 100, 100)
        );

        setLocationRelativeTo(this);
        timer.setRepeats(false);
        pack();
    }

    public ProgressDialog(JFrame parent, String title, String text){
        this(parent);
        setTitle(title);
        label.setText(text);
    }
    
    public final int showDialog(int timeout_ms){
        closeReason = CANCELLED;
        if (timeout_ms > 0){
            timer.setDelay(timeout_ms);
            timer.setInitialDelay(timeout_ms);
            timer.start();
        }
        setVisible(true);
        return closeReason;
    }
    
    public void hideDialog(){
        EventQueue.invokeLater(() -> {
            timer.stop();
            closeReason = OK;
            setVisible(false);
            dispose();
        });
    }

    public final void setText(String text){
        label.setText(text);
    }
}
